'' top.spin
'
' Read SD card files. Play multiple channels of WAV audio sounds.
'
'  top -------- wavPlayer[0] ---- fileReader[0] --------------- cardReader
'                 \ DAC cog_0                                /       \ SD cog
'                                                           /
'               wavPlayer[1] ---- fileReader[1] -----------/
'                 \ DAC cog_1                             /
'                                                        /
'               wavPlayer[2] ---- fileReader[2] --------/
'                 \ DAC cog_2                          /
'                                                     /
'               ...                                  /
'                                                   /
'               wavPlayer[5] ---- fileReader[5] ---/
'                 \ DAC cog_5                         
'
'                          
' Top starts an array of up to six wave players [0..5].
' Each wave player calls its own file reader to open an SD WAV file by name.
' Each wave player has its own sector data buffer and WAV parameter area, and
'   each player starts and operates its own DAC in its own cog.
' Each DAC cog has its own in-cog data cache so it can continue to play during
'   its wave player sector refresh.
' Each fileReader maintains its own individual file tracking parameters, and 
'   each reader has its own FAT buffer cache.
' All fileReaders share a single SD card reader.
'
' The card reader manages the card, copying data sectors from an SD card to
'   wave player hub sector buffers.
' The file readers manage the files, reading FAT16 filesystems, locating
'    files by name, then delivering next sectors on wave player request.
' The wavePlayers manage the play session, monitoring its user button for a start signal
'   then calling on its file reader to deliver the next data sector when needed. 
' Top repeatedly calls refresh on each wavePlayer, giving each the timeslot for button
' testing and/or sector requesting.


CON    

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000           

   CHANNELS = 3                  ' Enable from 1 to 6 CHANNELS
   
OBJ
                           
  wavPlayer[CHANNELS]: "WavPlayer"

                                   
PUB main | i 
                     
  repeat i from 0 to CHANNELS-1
    wavPlayer[i].init(i)
    
  repeat  
    repeat i from 0 to CHANNELS-1      
      wavPlayer[i].refresh
          


            